import React, { useRef, useEffect, useCallback } from 'react';
import { soundManager } from '../lib/sounds';

interface WheelProps {
  items: string[];
  spinning: boolean;
  background?: string;
  onSpinComplete: (winningItem: string) => void;
}

export function Wheel({ items, spinning, background, onSpinComplete }: WheelProps) {
  const canvasRef = useRef<HTMLCanvasElement>(null);
  const requestRef = useRef<number>();
  const rotationRef = useRef(0);
  const speedRef = useRef(0);
  const lastTickRef = useRef(0);
  const spinStartTimeRef = useRef(0);
  const backgroundImageRef = useRef<HTMLImageElement | null>(null);
  const targetRotationRef = useRef(0);
  const lastResultRef = useRef(-1);
  const isSpinningRef = useRef(false);

  // Load background image
  useEffect(() => {
    if (background) {
      const img = new Image();
      img.src = background;
      img.onload = () => {
        backgroundImageRef.current = img;
      };
    }
  }, [background]);

  // Initialize spin
  useEffect(() => {
    if (spinning && !isSpinningRef.current) {
      // Reset animation state
      isSpinningRef.current = true;
      speedRef.current = 5;
      spinStartTimeRef.current = performance.now();

      // Generate random number of full rotations (between 8 and 12)
      const fullRotations = 8 + Math.random() * 4;
      
      // Calculate slice size
      const sliceSize = (2 * Math.PI) / items.length;
      
      // Find a random slice that's different from the last result
      let randomSlice;
      do {
        randomSlice = Math.floor(Math.random() * items.length);
      } while (randomSlice === lastResultRef.current && items.length > 1);
      
      lastResultRef.current = randomSlice;
      
      // Calculate target rotation to land on the selected slice
      const targetSlice = items.length - 1 - randomSlice;
      const currentRotation = rotationRef.current % (2 * Math.PI);
      targetRotationRef.current = rotationRef.current + (fullRotations * 2 * Math.PI) + (targetSlice * sliceSize) - currentRotation;
      
      // Play sound effects
      soundManager.playThemedSound('spin', 0.5);
      if (navigator.vibrate) {
        navigator.vibrate([15, 50, 15]);
      }
    } else if (!spinning) {
      isSpinningRef.current = false;
    }
  }, [spinning, items.length]);

  // Draw wheel
  const drawWheel = useCallback((ctx: CanvasRenderingContext2D, centerX: number, centerY: number, radius: number) => {
    items.forEach((item, index) => {
      const slice = (2 * Math.PI) / items.length;
      const angle = index * slice;

      ctx.save();
      ctx.beginPath();
      ctx.moveTo(centerX, centerY);
      ctx.arc(centerX, centerY, radius, angle + rotationRef.current, angle + slice + rotationRef.current);
      ctx.closePath();

      const hue = (index * 360) / items.length;
      ctx.shadowBlur = spinning ? 30 : 20;
      ctx.shadowColor = `hsl(${hue}, 100%, 60%)`;
      ctx.fillStyle = `hsl(${hue}, 70%, ${spinning ? '30%' : '20%'})`;
      ctx.fill();

      ctx.translate(centerX, centerY);
      ctx.rotate(angle + rotationRef.current + slice / 2);
      ctx.textAlign = 'right';
      ctx.fillStyle = '#fff';
      
      const isMobile = window.innerWidth < 768;
      const maxTextLength = Math.max(...items.map(item => item.length));
      const fontSize = Math.min(
        radius * (isMobile ? 0.45 : 0.35),
        (radius * 0.95) / maxTextLength,
        isMobile ? 64 : 48
      );
      
      ctx.font = `bold ${fontSize}px sans-serif`;
      ctx.strokeStyle = 'rgba(0,0,0,0.7)';
      ctx.lineWidth = fontSize * 0.15;
      
      const textX = radius - (radius * 0.12);
      const textY = 0;
      
      const maxWidth = radius * 0.9;
      const words = item.split(' ');
      
      if (words.length > 1 && ctx.measureText(item).width > maxWidth) {
        const lines = [];
        let currentLine = words[0];
        
        for (let i = 1; i < words.length; i++) {
          const testLine = currentLine + ' ' + words[i];
          if (ctx.measureText(testLine).width <= maxWidth) {
            currentLine = testLine;
          } else {
            lines.push(currentLine);
            currentLine = words[i];
          }
        }
        lines.push(currentLine);
        
        const lineHeight = fontSize * 1.2;
        const totalHeight = lineHeight * (lines.length - 1);
        let y = -totalHeight / 2;
        
        lines.forEach(line => {
          ctx.strokeText(line, textX, y);
          ctx.fillText(line, textX, y);
          y += lineHeight;
        });
      } else {
        ctx.strokeText(item, textX, textY);
        ctx.fillText(item, textX, textY);
      }
      
      ctx.restore();
    });
  }, [items, spinning]);

  // Animation loop
  useEffect(() => {
    const canvas = canvasRef.current;
    if (!canvas) return;

    const rect = canvas.getBoundingClientRect();
    canvas.width = rect.width * window.devicePixelRatio;
    canvas.height = rect.height * window.devicePixelRatio;

    const ctx = canvas.getContext('2d');
    if (!ctx) return;

    ctx.scale(window.devicePixelRatio, window.devicePixelRatio);

    let lastTimestamp = 0;
    const animate = (timestamp: number) => {
      if (!canvas || !ctx) return;

      // Calculate delta time for smooth animation
      const deltaTime = lastTimestamp ? (timestamp - lastTimestamp) / 1000 : 0;
      lastTimestamp = timestamp;

      ctx.clearRect(0, 0, canvas.width, canvas.height);
      
      const centerX = rect.width / 2;
      const centerY = rect.height / 2;
      const radius = Math.min(centerX, centerY) * 0.9;

      // Draw background
      if (backgroundImageRef.current) {
        ctx.save();
        ctx.globalAlpha = 0.2;
        ctx.beginPath();
        ctx.arc(centerX, centerY, radius, 0, Math.PI * 2);
        ctx.clip();
        const img = backgroundImageRef.current;
        const scale = Math.max(rect.width / img.width, rect.height / img.height);
        const w = img.width * scale;
        const h = img.height * scale;
        const x = (rect.width - w) / 2;
        const y = (rect.height - h) / 2;
        ctx.drawImage(img, x, y, w, h);
        ctx.restore();
      }

      // Draw wheel segments
      drawWheel(ctx, centerX, centerY, radius);

      // Draw pointer
      ctx.save();
      ctx.beginPath();
      ctx.moveTo(centerX + radius - 5, centerY);
      ctx.lineTo(centerX + radius + 15, centerY - 15);
      ctx.lineTo(centerX + radius + 15, centerY + 15);
      ctx.closePath();
      ctx.fillStyle = spinning ? '#fff' : '#f0f0f0';
      ctx.shadowBlur = spinning ? 25 : 15;
      ctx.shadowColor = spinning ? '#ff00ff' : '#ff00cc';
      ctx.fill();
      ctx.restore();

      if (isSpinningRef.current) {
        const now = performance.now();
        const spinDuration = now - spinStartTimeRef.current;
        
        if (now - lastTickRef.current > (1000 / speedRef.current)) {
          soundManager.playTick(Math.min(0.3, speedRef.current * 0.05));
          lastTickRef.current = now;
        }

        // Start slowing down after 4 seconds
        if (spinDuration > 4000) {
          speedRef.current *= 0.98;
        }

        // Update rotation based on current speed and delta time
        const rotationDelta = speedRef.current * deltaTime * 2;
        rotationRef.current += rotationDelta;

        if (speedRef.current < 0.01 || rotationRef.current >= targetRotationRef.current) {
          // Stop the animation
          speedRef.current = 0;
          rotationRef.current = targetRotationRef.current;
          isSpinningRef.current = false;
          
          // Play win sound and vibrate
          soundManager.playWin();
          if (navigator.vibrate) {
            navigator.vibrate(100);
          }

          // Calculate winning item
          const normalizedRotation = ((rotationRef.current % (2 * Math.PI)) + (2 * Math.PI)) % (2 * Math.PI);
          const slice = (2 * Math.PI) / items.length;
          const winningIndex = items.length - 1 - Math.floor(normalizedRotation / slice);
          const actualIndex = winningIndex % items.length;
          onSpinComplete(items[actualIndex]);
        }
      }

      requestRef.current = requestAnimationFrame(animate);
    };

    requestRef.current = requestAnimationFrame(animate);

    return () => {
      if (requestRef.current) {
        cancelAnimationFrame(requestRef.current);
      }
    };
  }, [items, spinning, onSpinComplete, drawWheel]);

  return (
    <canvas
      ref={canvasRef}
      className="w-full h-full touch-manipulation active:scale-[0.99] transition-transform"
      style={{ 
        touchAction: 'manipulation',
        width: '100%',
        height: '100%'
      }}
    />
  );
}